# A TCP server accepting multiple connections, each having his own send / recv
# threads

import socket
import thread
from defaultRecvHandler import defaultRecvHandler
from time import sleep

# Stores information about a client and provides thread safe input / output
# buffers
class client:
  def __init__(self, id, socket, addr, lock):
    self.id = id
    self.socket = socket
    self.addr = addr
    self.sendBuffer = []
    self.sendFlag = 0
    self.sendLock = lock
    self.stopRecvThread = 0
    self.stopSendThread = 0
    self.recvThreadStopped = 0
    self.sendThreadStopped = 0

  def send(self, data):
    self.sendLock.acquire()
    self.sendBuffer.append(data)
    self.sendFlag = 1
    self.sendLock.release()

  def flush(self):
    self.sendLock.acquire()
    res = self.sendBuffer
    self.sendBuffer = []
    self.sendFlag = 0
    self.sendLock.release()    
    return res

# A server supporting multiple connections and sends / receives to / from them asynchronously
class TCPServer:
  def __init__(self, ip, port):
    self.lastId = 0
    self.addr = (ip, port)
    self.granularity = 1.0 / 60.0
    self.clients = {}
    self.recvBufferSize = 2048  
    self.recvHandler = defaultRecvHandler
    # Create the server socket and bind it
    self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    self.socket.bind(self.addr)
    
  def start(self):
    self.stopAcceptThread = 0
    self.stopRecvThread = 0
    self.stopSendThread = 0
    thread.start_new_thread(self.__accept, ())

  def stop(self):
    self.stopAcceptThread = 1
    self.stopRecvThread = 1 
    self.stopSendThread = 1

  def send(self, clientId, data):
    self.clients[clientId].send(data)

  def sendAll(self, data):
    for id in self.clients:
      self.clients[id].send(data)

  def printClients(self):
    i = 0
    for id in self.clients:
      print '%i: %s' % (i, self.clients[id].addr)
      i += 1

  ### Private methods ### 
  
  # This thread listens for incoming TCP connections and establishes asynchronous bidirectionnal links with them
  def __accept(self):
    while not self.stopAcceptThread:
      self.socket.listen(1)
      socket, addr = self.socket.accept()
      # Got a new connection, start new receiving and sending threads for it
      id = self.lastId + 1
      self.lastId = id
      self.clients[id] = client(id, socket, addr, thread.allocate_lock())
      thread.start_new_thread(self.__recv, (id, socket))
      thread.start_new_thread(self.__send, (id, socket))
      sleep(self.granularity)
  
  def __recv(self, id, socket):
    try:
      # Main receiving loop
      while (not self.clients[id].stopRecvThread) and (not self.stopRecvThread):
        try:
          data = socket.recv(self.recvBufferSize)
        except: break
        if data:
          a = self.recvHandler()
          a.handle(data)
        sleep(self.granularity)
      
      # Remove client object
      del self.clients[id]
    except:
      a = 0
      
  def __send(self, id, socket):
    try:
      while not self.clients[id].stopSendThread:
        if self.clients[id].sendFlag:
          sendBuffer = self.clients[id].flush()
          try:
            for data in sendBuffer:
              socket.sendto(data, self.clients[id].addr)
          except: break
        sleep(self.granularity)

      # Remove client object    
      del self.clients[id]
    except:
      a = 0      
